//*******************************************************
// VRML++ Preprocessor
// Copyright (C) 1996 Stephan Diehl
// 
// extends the VRML 2.0 Parser of Bell and Woods,
// (see Copyright message below)
// 
// extensions are enclosed in #ifdef STED ... #endif
//            or comments /*<< STED */ ... /* STED >>*/
//
//*******************************************************
// Major Extensions 
//   * building a parse tree while parsing
//   * checking whether USED nodes are DEFined
//   * adding productions for VRML++ features
//   * improved type checking 
//   * conversion of VRML++ features into VRML 2.0
//   * unparsing of parse tree
//*******************************************************

/**************************************************
 * VRML 2.0, Draft 2 Parser
 * Copyright (C) 1996 Silicon Graphics, Inc.
 *
 * Author(s)    : Gavin Bell
 *                Daniel Woods (first port)
 **************************************************
 */

#include <iostream.h>
#include <stdio.h>
#include "vrmlnode.h"

extern int yyparse();
extern void yyResetLineNumber();
extern int yydebug;
extern int yy_flex_debug;
extern FILE *yyin;

extern ParseTreeNode* parsetree;

char* whichscript="vrmlscript";
// char* initfile="standardNodes.wrl";
char* initfile="standardClasses.vpp";

char* linebuffer;

usage(char* name)
{ cerr << "Usage: " << name << " [-js] [-init initfile] filename \n\n";
}

main(int argc, char **argv)
{
    //
    // Read in PROTO declarations for all of the standard nodes:
    //
#ifdef STED
   char *filename;
   int i;
   linebuffer= (char*) malloc(2000);
   linebuffer[0]='\0';
   if (argc==1)
     { usage(argv[0]);
       exit(1);
     }
   for(i = 1; (i < argc) && (argv[i][0] == '-'); i++)
     { if (strcmp(argv[i]+1,"js")==0) 
       { whichscript="javascript";    
       }
      else 
       if (strcmp(argv[i]+1,"init")==0) 
        { initfile=argv[++i];
        }
       else { usage(argv[0]); exit(1); }
     }
    if (argv[i]) 
      { filename=argv[i]; }
    else { usage(argv[0]); exit(1); }
#endif
    yydebug = 0;
    yy_flex_debug = 0;
    FILE *standardNodes = fopen(initfile, "r");
    if (standardNodes == NULL) {
    	cerr << "Error, couldn't open " << initfile << " file";
        exit(1);
    }
    yyin = standardNodes;
    yyparse();
    yyin = NULL;
    yyResetLineNumber();
    fclose(standardNodes);
    int flags = 0;
#ifdef STED
 
#else
	if (argc > 1) {
    	flags = atoi(argv[1]);
    }
    yydebug = flags & 1;
    yy_flex_debug = flags & 2;

    char filename[100];
    cerr << "Enter test file name: ";
    cin >> filename;
#endif
    yyin = fopen(filename, "r");

    if (yyin == NULL) {
    	cerr << "Error, could not open file\n";
        exit(1);
    }
    // For this little test application, pushing and popping the node
    // namespace isn't really necessary.  But each VRML .wrl file is
    // a separate namespace for PROTOs (except for things predefined
    // in the spec), and pushing/popping the namespace when reading each
    // file is a good habit to get into:
    VrmlNodeType::pushNameSpace();
    parsetree=NULL;
    yyparse();
    if (parsetree) parsetree=convert(parsetree);
    cout << "#VRML V2.0 utf8\n\n";
    cout << "# This VRML 2.0 file was generated from the VRML++ file " << filename << "\n";
    cout << "# using the VRML++ Preprocessor, (c) Stephan Diehl, 1997\n\n";
    unparse(parsetree);
    VrmlNodeType::popNameSpace();


    return(0);
}
